
local squaredMaze = require 'squaredMaze'

local examples = {
	{n="Backtracker",v= require 'algorithms.backtracker'},
	{n="Kruskals",v= require 'algorithms.kruskals'},
	{n="Growingtree",v= require 'algorithms.growingtree'},
	--division	= require 'algorithms.division',
	{n="Prims",v= require 'algorithms.prims'},
	{n="Drunkard",v= require 'algorithms.drunkard'},
	{n="Binary",v= require 'algorithms.binary'},
	{n="Braid\nPrims",v= require 'algorithms.braidPrims'},
	{n="Braid\nBacktracker",v= require 'algorithms.braidBacktracker'},
	{n="Growing\nBacktracker",v= require 'algorithms.growingBacktracker'},
	{n="Braid\nKruskals",v= require 'algorithms.braidKruskals'},
}

table.sort(examples,function(a,b) return a.n < b.n end)

local kc = 'upupdowndownleftrightleftrightba'
local ck = 1

local version = tonumber(string.sub(love._version,1,3))

local algorithm = examples[1]
local currentAlg = 1
local scroll = 0
local scrolling = 0
local speed = 390
local speedHeld = false
local speedOffset = 0
local running = true

local tileset = love.graphics.newImage('exampleTileset.png')
local tileWidth = 16
local tileHeight = 16
local tileQuads = {}
local tileBatch

local maze
local width = math.floor(600/tileWidth)
local height = math.floor(600/tileHeight)

local function lerp(a,b,t)
	return a + (b - a) * t
end

local function delerp(a,b,c)
	return (c - a)/(b - a)
end

local function mazeRawCallback(x,y,v)
	-- bartbes told me not to guess the ID but I did not listen.
	if version >= 0.9 then
		tileBatch:set(y*width+x-1,tileQuads[v],x*tileWidth,y*tileHeight,0,1,1,tileWidth,tileHeight)
	else
		tileBatch:setq(y*width+x-1,tileQuads[v],x*tileWidth,y*tileHeight,0,1,1,tileWidth,tileHeight)
	end
end

local function resetMap()
	tileBatch:clear()
	for i = 1, height+1 do
		for j = 1, width do
			if version >= 0.9 then
				tileBatch:add(tileQuads[0],j*tileWidth,i*tileHeight,0,1,1,tileWidth,tileHeight)
			else
				tileBatch:addq(tileQuads[0],j*tileWidth,i*tileHeight,0,1,1,tileWidth,tileHeight)
			end
		end
	end
	collectgarbage('collect')
end

local function reset()
	maze = squaredMaze.newMaze(width,height)
	maze.setRawCallback = mazeRawCallback
	resetMap()
	algorithm.v.reset(maze,width,height)
end

function love.load()
	love.graphics.setNewFont(16)
	for i = 0,15 do
		tileQuads[i] = love.graphics.newQuad(math.mod(i,4)*tileWidth,math.floor(i/4)*tileHeight,tileWidth,tileHeight,tileset:getWidth(),tileset:getHeight())
	end
	tileBatch = love.graphics.newSpriteBatch(tileset,width*height*2)
	reset()
	resetMap()
end

function love.keypressed(k)
	if string.sub(kc,ck,ck+string.len(k)-1) == k then
		ck=ck+string.len(k)
		if ck > string.len(kc) then
			reset()
			ck=1
		end
	else ck=1 end
end

function love.mousepressed(x,y,b)
	if b == 'l' then
		local j = {}
		for i,v in ipairs(examples) do
			j[#j+1] = i
		end
		if y > 530 and y < 570 and x > lerp(50,150,delerp(60,720,speed)) - 20 and x < lerp(50,150,delerp(60,720,speed)) + 20 then
			speedHeld = true
			speedOffset = lerp(50,150,delerp(60,720,speed))-x
		elseif x > 150 and x < 190 and y > lerp(10,410,delerp(0,#j*100-500,scroll)) and y < lerp(90,490,delerp(0,#j*100-500,scroll)) then
			scrolling = 2
			scrollOffset = lerp(10,410,delerp(0,#j*100-500,scroll))-y
		elseif math.mod(y+scroll,100) > 10 and math.mod(y+scroll,100) < 90 and x > 10 and x < 140 and y < 500 then
			local i = math.ceil((y+scroll)/100)
			if i <= #j then
				algorithm = examples[j[i]]
				currentAlg = j[i]
				reset()
			end
		end
	elseif b == 'r' then
		if y > 0 and y < 500 and x > 0 and x < 200 then
			scrolling = 1
		end
	end
end

function love.mousereleased(x,y,b)
	if b == 'l' then
		speedHeld = false
		if scrolling == 2 then scrolling = 0 end
	elseif b == 'r' then
		if scrolling == 1 then scrolling = 0 end
	end
end

local lastY
local t = 0

function love.update(dt)
	if running then
		t=t+dt
		if t > 1/speed*algorithm.v.speed then
			for i = 1/speed*algorithm.v.speed, t, 1/speed*algorithm.v.speed do
				algorithm.v.update(maze)
			end
			t=t-1/speed*algorithm.v.speed
			if t > 1/speed*algorithm.v.speed then
				t=0
			end
		end
	else
		t=0
	end
	local e = 0
	for i,v in ipairs(examples) do
		e=e+1
	end
	if scrolling == 1 then
		scroll = math.max(0,math.min(e*100-500,scroll - love.mouse.getY()+lastY))
	elseif scrolling == 2 then
		scroll = math.max(0,math.min(e*100-500,lerp(0,e*100-500,delerp(10,410,scrollOffset+love.mouse.getY()))))
	end
	lastY=love.mouse.getY()
	if speedHeld then
		speed = math.max(60,math.min(720,lerp(60,720,delerp(50,150,speedOffset+love.mouse.getX()))))
	end
end

function love.draw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(tileBatch,200 + math.mod(600,16)/2,math.mod(600,16)/2)
	love.graphics.line(200,0,200,600)
	local ax = lerp(50,150,delerp(60,720,speed))
	local e = 0
	for i,v in ipairs(examples) do
		e=e+1
	end
	local mx,my = love.mouse.getPosition()
	love.graphics.print('Speed',5,575)
	if (mx > ax-20 and mx < ax+20 and my > 530 and my < 570) or speedHeld then
		if speedHeld then
			love.graphics.setColor(255,255,255)
		else
			love.graphics.setColor(127,127,127)
		end
		love.graphics.rectangle('fill',ax-20,530,40,40)
		love.graphics.setColor(255,255,255)
	else
		love.graphics.rectangle('line',ax-20,530,40,40)
	end
	love.graphics.line(math.min(ax-20,30),550,ax-20,550)
	love.graphics.line(math.max(ax+20,170),550,ax+20,550)

	local by = lerp(10,410,delerp(0,e*100-500,scroll))
	love.graphics.line(170,math.min(by,10),170,by)
	love.graphics.line(170,math.max(by+80,490),170,by+80)
	if (mx > 150 and mx < 190 and my > by and my < by+80) or scrolling == 2 then
		if scrolling == 2 then
			love.graphics.setColor(255,255,255)
		else
			love.graphics.setColor(127,127,127)
		end
		love.graphics.rectangle('fill',150,lerp(10,410,delerp(0,e*100-500,scroll)),40,80)
	else
		love.graphics.rectangle('line',150,lerp(10,410,delerp(0,e*100-500,scroll)),40,80)
	end
	local a = 0
	local b = -1
	if math.mod(my+scroll,100) > 10 and math.mod(my+scroll,100) < 90 and mx > 10 and mx < 140 and my < 500 then
		b = math.floor((my+scroll)/100)
	end
	love.graphics.setScissor(0,0,200,500)
	for i,v in ipairs(examples) do
		local w,l = love.graphics.getFont():getWrap(v.n, love.graphics.getFont():getWidth(v.n))
		if currentAlg == i then
			love.graphics.setColor(255,255,255)
			love.graphics.rectangle('fill',10,100*a+10-scroll,130,80)
			love.graphics.setColor(0,0,0)
			love.graphics.print(v.n,75,100*a+50-scroll,0,1,1,math.floor(w/2),math.floor(l*love.graphics.getFont():getHeight()/2))
		elseif a == b then
			love.graphics.setColor(64,64,64)
			love.graphics.rectangle('fill',10,100*a+10-scroll,130,80)
			love.graphics.setColor(255,255,255)
			love.graphics.rectangle('line',10,100*a+10-scroll,130,80)
			love.graphics.print(v.n,75,100*a+50-scroll,0,1,1,math.floor(w/2),math.floor(l*love.graphics.getFont():getHeight()/2))
		else
			love.graphics.setColor(255,255,255)
			love.graphics.rectangle('line',10,100*a+10-scroll,130,80)
			love.graphics.print(v.n,75,100*a+50-scroll,0,1,1,math.floor(w/2),math.floor(l*love.graphics.getFont():getHeight()/2))
		end
		a=a+1
	end
	love.graphics.setScissor()
end
